@@ -2,7 +2,7 @@
  * FileTranscoder interface for MP3FS
  *
  * Copyright (C) 2006-2008 David Collett
- * Copyright (C) 2008-2011 Kristofer Henriksson
+ * Copyright (C) 2008-2012 Kristofer Henriksson
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,8 +36,6 @@
 
 #include "transcode.h"
 
-extern struct mp3fs_params params;
-
 /* Functions for dealing with the buffer */
 
 /*
@@ -107,8 +105,8 @@ struct id3_frame *make_frame(const char *name, const char *data) {
 /* return a vorbis comment tag */
 const char *get_tag(const FLAC__StreamMetadata *metadata, const char *name) {
     int idx;
-    FLAC__StreamMetadata_VorbisComment *comment;
-    comment = (FLAC__StreamMetadata_VorbisComment *)&metadata->data;
+    const FLAC__StreamMetadata_VorbisComment *comment;
+    comment = &metadata->data.vorbis_comment;
     idx = FLAC__metadata_object_vorbiscomment_find_entry_from(metadata, 0,
                                                               name);
     if (idx<0) return NULL;
@@ -126,11 +124,11 @@ void set_tag(const FLAC__StreamMetadata *metadata, struct id3_tag *id3tag,
 /* set id3 picture tag from FLAC picture block */
 void set_picture_tag(const FLAC__StreamMetadata *metadata,
                      struct id3_tag *id3tag) {
-    FLAC__StreamMetadata_Picture *picture;
+    const FLAC__StreamMetadata_Picture *picture;
     struct id3_frame *frame;
     id3_ucs4_t       *ucs4;
 
-    picture = (FLAC__StreamMetadata_Picture *)&metadata->data;
+    picture = &metadata->data.picture;
 
     /*
      * There hardly seems a point in separating out these into a different
@@ -216,7 +214,7 @@ write_cb(const FLAC__StreamDecoder *decoder, const FLAC__Frame *frame,
         lbuf[i] = buffer[0][i] <<
             (sizeof(int)*8 - frame->header.bits_per_sample);
         // ignore rbuf for mono sources
-        if (trans->info.channels > 1) {
+        if (frame->header.channels > 1) {
             rbuf[i] = buffer[1][i] <<
                 (sizeof(int)*8 - frame->header.bits_per_sample);
         }
@@ -243,22 +241,22 @@ static void meta_cb(const FLAC__StreamDecoder *decoder,
     char tmpstr[10];
     float dbgain = 0;
     float filegainref;
+    FLAC__StreamMetadata_StreamInfo info;
     struct transcoder* trans = (struct transcoder*)client_data;
 
     switch (metadata->type) {
         case FLAC__METADATA_TYPE_STREAMINFO:
-            memcpy(&trans->info, &metadata->data,
-                   sizeof(FLAC__StreamMetadata_StreamInfo));
+            info = metadata->data.stream_info;
 
             /* set the length in the id3tag */
             snprintf(tmpstr, 10, "%" PRIu64,
-                trans->info.total_samples*1000/trans->info.sample_rate);
+                info.total_samples*1000/info.sample_rate);
             id3_tag_attachframe(trans->id3tag, make_frame("TLEN", tmpstr));
 
             /* Use the data in STREAMINFO to set lame parameters. */
-            lame_set_num_samples(trans->encoder, trans->info.total_samples);
-            lame_set_in_samplerate(trans->encoder, trans->info.sample_rate);
-            lame_set_num_channels(trans->encoder, trans->info.channels);
+            lame_set_num_samples(trans->encoder, info.total_samples);
+            lame_set_in_samplerate(trans->encoder, info.sample_rate);
+            lame_set_num_channels(trans->encoder, info.channels);
 
             break;
         case FLAC__METADATA_TYPE_VORBIS_COMMENT:
@@ -360,7 +358,6 @@ static void meta_cb(const FLAC__StreamDecoder *decoder,
 
 struct transcoder* transcoder_new(char *flacname) {
     struct transcoder* trans;
-    unsigned long numframes;
     uint8_t* write_ptr;
 
     mp3fs_debug("Creating transcoder object for %s", flacname);
@@ -421,12 +418,10 @@ struct transcoder* transcoder_new(char *flacname) {
 
     /*
      * Process metadata. This will fill in the id3tag and the remaining
-     * lame parameters.
+     * lame parameters. If the function fails, the FLAC is invalid.
      */
-    FLAC__stream_decoder_process_until_end_of_metadata(trans->decoder);
-
-    /* If sample rate hasn't been set yet, the FLAC is invalid. */
-    if (!trans->info.sample_rate) {
+    if (!FLAC__stream_decoder_process_until_end_of_metadata(trans->decoder)) {
+        mp3fs_debug("FLAC is invalid.");
         goto lame_fail;
     }
 
@@ -467,15 +462,18 @@ struct transcoder* transcoder_new(char *flacname) {
 
     // store v1 tag
     id3_tag_options(trans->id3tag, ID3_TAG_OPTION_ID3V1, ~0);
-    id3_tag_render(trans->id3tag, (id3_byte_t *)trans->id3v1tag);
+    id3_tag_render(trans->id3tag, trans->id3v1tag);
 
     mp3fs_debug("Tag written.");
 
-    // id3v2 + lame stuff + mp3 data + id3v1
-    numframes = divideround(trans->info.total_samples, 1152) + 2;
-    trans->totalsize = trans->buffer.pos
-        + divideround((long long)numframes*144*params.bitrate*10,
-                      trans->info.sample_rate/100) + 128;
+    /*
+     * Properly calculate final file size. This is the sum of the size of
+     * ID3v2, ID3v1, and raw MP3 data.
+     * Cast to 64-bit int to avoid overflow
+     */
+    trans->totalsize = trans->buffer.pos + 128
+        + (uint64_t)lame_get_totalframes(trans->encoder)*144*params.bitrate*10
+        / (lame_get_out_samplerate(trans->encoder)/100);
 
     id3_tag_delete(trans->id3tag);
     return trans;
@@ -502,6 +500,10 @@ trans_fail:
 /* Read some bytes into the internal buffer and into the given buffer. */
 
 int transcoder_read(struct transcoder* trans, char* buff, int offset, int len) {
+    /* Client asked for more data than exists. */
+    if (offset > trans->totalsize) {
+        return 0;
+    }
     if (offset+len > trans->totalsize) {
         len = trans->totalsize - offset;
     }
@@ -577,13 +579,11 @@ int transcoder_finish(struct transcoder* trans) {
         lame_close(trans->encoder);
         trans->encoder = NULL;
 
-        if (trans->buffer.pos + 128 != trans->totalsize) {
-            // write the id3v1 tag, always 128 bytes from end
-            mp3fs_debug("Something went wrong with file size calculation: "
-                  "off by %lu", trans->buffer.pos + 128 - trans->totalsize);
-            trans->buffer.pos = trans->totalsize - 128;
-        }
-        buffer_write(&trans->buffer, (uint8_t*)trans->id3v1tag, 128);
+        /* Write the ID3v1 tag, always 128 bytes from end. */
+        mp3fs_debug("Finishing file. Correct size: %lu, computed size: %lu",
+                    trans->buffer.pos + 128, trans->totalsize);
+        trans->buffer.pos = trans->totalsize - 128;
+        buffer_write(&trans->buffer, trans->id3v1tag, 128);
         len += 128;
     }
 
