@@ -2,7 +2,7 @@
  * FileTranscoder interface for MP3FS
  *
  * Copyright (C) 2006-2008 David Collett
- * Copyright (C) 2008-2011 Kristofer Henriksson
+ * Copyright (C) 2008-2012 Kristofer Henriksson
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +19,9 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#define FUSE_USE_VERSION 26
+
+#include <fuse.h>
 #include <FLAC/metadata.h>
 #include <FLAC/stream_decoder.h>
 #include <lame/lame.h>
@@ -29,14 +32,17 @@
 #define BUFSIZE 2 * FLAC_BLOCKSIZE
 
 /* Global program parameters */
-struct mp3fs_params {
+extern struct mp3fs_params {
     const char *basepath;
     unsigned int bitrate;
     unsigned int quality;
     int debug;
     int gainmode;
     float gainref;
-};
+} params;
+
+/* Fuse operations struct */
+extern struct fuse_operations mp3fs_ops;
 
 #define mp3fs_debug(f, ...) syslog(LOG_DEBUG, f, ## __VA_ARGS__)
 #define mp3fs_info(f, ...) syslog(LOG_INFO, f, ## __VA_ARGS__)
@@ -55,14 +61,10 @@ struct transcoder {
     unsigned long totalsize;
 
     struct id3_tag *id3tag;
-    char id3v1tag[128];
+    id3_byte_t id3v1tag[128];
 
     FLAC__StreamDecoder *decoder;
     lame_global_flags *encoder;
-
-    FLAC__StreamMetadata_StreamInfo info;
-
-    unsigned char mp3buf[BUFSIZE];
 };
 
 struct transcoder* transcoder_new(char *flacname);
